==> xmame-0.36.1.tgz: xmame-0.36.1/src/mess/vidhrdw/trs80.c <==
/***************************************************************************

  trs80.c

  Functions to emulate the video hardware of the TRS80.

***************************************************************************/

#include "driver.h"
#include "palette.h"
#include "mess/systems/trs80.h"

#define FW  TRS80_FONT_W
#define FH  TRS80_FONT_H

extern UINT8 port_ff;
extern void trs80_shutdown_machine(void);

/***************************************************************************
  Statics for this module
***************************************************************************/
static int color = 0;

/***************************************************************************
  Start the video hardware emulation.
***************************************************************************/
int trs80_vh_start(void)
{
	if (generic_vh_start() != 0)
		return 1;

	return 0;
}

void trs80_vh_stop(void)
{
	generic_vh_stop();
}

/***************************************************************************
  Draw the game screen in the given osd_bitmap.
  Do NOT call osd_update_display() from this function,
  it will be called by the main emulation engine.
***************************************************************************/
void trs80_vh_screenrefresh(struct osd_bitmap *bitmap, int full_refresh)
{
/* Special translation if video RAM with only 7 bits is present
 * I don't know if it's entirely correct, but it's close ;-)
 */
static UINT8 translate_videoram[2][256] = {
	{
		0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f,
		0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1a,0x1b,0x1c,0x1d,0x1e,0x1f,
		0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2a,0x2b,0x2c,0x2d,0x2e,0x2f,
		0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,0x3d,0x3e,0x3f,
		0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,
		0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5a,0x5b,0x5c,0x5d,0x5e,0x5f,
		0x60,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6a,0x6b,0x6c,0x6d,0x6e,0x6f,
		0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7a,0x7b,0x7c,0x7d,0x7e,0x7f,
		0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8a,0x8b,0x8c,0x8d,0x8e,0x8f,
		0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9a,0x9b,0x9c,0x9d,0x9e,0x9f,
		0xa0,0xa1,0xa2,0xa3,0xa4,0xa5,0xa6,0xa7,0xa8,0xa9,0xaa,0xab,0xac,0xad,0xae,0xaf,
		0xb0,0xb1,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xba,0xbb,0xbc,0xbd,0xbe,0xbf,
		0xc0,0xc1,0xc2,0xc3,0xc4,0xc5,0xc6,0xc7,0xc8,0xc9,0xca,0xcb,0xcc,0xcd,0xce,0xcf,
		0xd0,0xd1,0xd2,0xd3,0xd4,0xd5,0xd6,0xd7,0xd8,0xd9,0xda,0xdb,0xdc,0xdd,0xde,0xdf,
		0xe0,0xe1,0xe2,0xe3,0xe4,0xe5,0xe6,0xe7,0xe8,0xe9,0xea,0xeb,0xec,0xed,0xee,0xef,
		0xf0,0xf1,0xf2,0xf3,0xf4,0xf5,0xf6,0xf7,0xf8,0xf9,0xfa,0xfb,0xfc,0xfd,0xfe,0xff
	},
	{
		0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,
		0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5a,0x5b,0x5c,0x5d,0x5e,0x5f,
        0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2a,0x2b,0x2c,0x2d,0x2e,0x2f,
        0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,0x3d,0x3e,0x3f,
		0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,
        0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5a,0x5b,0x5c,0x5d,0x5e,0x5f,
		0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,
        0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5a,0x5b,0x5c,0x5d,0x5e,0x5f,
        0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8a,0x8b,0x8c,0x8d,0x8e,0x8f,
        0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9a,0x9b,0x9c,0x9d,0x9e,0x9f,
        0xa0,0xa1,0xa2,0xa3,0xa4,0xa5,0xa6,0xa7,0xa8,0xa9,0xaa,0xab,0xac,0xad,0xae,0xaf,
        0xb0,0xb1,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xba,0xbb,0xbc,0xbd,0xbe,0xbf,
        0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8a,0x8b,0x8c,0x8d,0x8e,0x8f,
        0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9a,0x9b,0x9c,0x9d,0x9e,0x9f,
        0xa0,0xa1,0xa2,0xa3,0xa4,0xa5,0xa6,0xa7,0xa8,0xa9,0xaa,0xab,0xac,0xad,0xae,0xaf,
        0xb0,0xb1,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xba,0xbb,0xbc,0xbd,0xbe,0xbf}
};

	int offs = 0;
	int translate = (input_port_0_r(0) & 0x40) ? 1 : 0;

	if( readinputport(0) & 0x08 )
        cpu_set_nmi_line(0, PULSE_LINE);

    /* check for changed color settings */
	if( color != (readinputport(0) & 3) )
    {
		color = readinputport(0) & 3;
		full_refresh = 1;
    }

    /* draw entire scrbitmap because of usrintrf functions
	 * called osd_clearbitmap or color change / scanline change
	 */
	if( palette_recalc() || full_refresh )
		memset(dirtybuffer, 1, videoram_size);

	/* do we have double width characters enabled ? */
	if( port_ff & 0x08 )
	{
		/* for every second character in the Video RAM, check if it has
		   been modified since last time and update it accordingly. */
		for (offs = videoram_size - 2; offs >= 0; offs -= 2)
		{
			if (dirtybuffer[offs])
			{
				int sx, sy;

				dirtybuffer[offs] = 0;
				sx = (offs % 64) * FW * 2;
				sy = (offs / 64) * FH;
				drawgfx(bitmap, Machine->gfx[1], translate_videoram[translate][videoram[offs]],
					color,0,0, sx,sy, &Machine->drv->visible_area, TRANSPARENCY_NONE,0);
			}
		}
	}
	else
	/* normal width characters */
	{
		/* for every character in the Video RAM, check if it has
		* been modified since last time and update it accordingly. */
		for (offs = videoram_size - 1; offs >= 0; offs--)
		{
			if (dirtybuffer[offs])
			{
				int sx, sy;

                dirtybuffer[offs] = 0;
				sx = (offs % 64) * FW;
				sy = (offs / 64) * FH;
				drawgfx(bitmap, Machine->gfx[0], translate_videoram[translate][videoram[offs]],
					color,0,0, sx,sy, &Machine->drv->visible_area, TRANSPARENCY_NONE,0);
            }
		}
	}
}

==> xmame-0.37b4.1.tgz: xmame-0.37b4.1/src/mess/vidhrdw/trs80.c <==
/***************************************************************************

  trs80.c

  Functions to emulate the video hardware of the TRS80.

***************************************************************************/

#include "driver.h"
#include "palette.h"
#include "mess/systems/trs80.h"

#define FW  TRS80_FONT_W
#define FH  TRS80_FONT_H

extern UINT8 port_ff;
extern void trs80_shutdown_machine(void);

/***************************************************************************
  Statics for this module
***************************************************************************/
static int color = 0;

/***************************************************************************
  Start the video hardware emulation.
***************************************************************************/
int trs80_vh_start(void)
{
	if (generic_vh_start() != 0)
		return 1;

	return 0;
}

void trs80_vh_stop(void)
{
	generic_vh_stop();
}

/***************************************************************************
  Draw the game screen in the given osd_bitmap.
  Do NOT call osd_update_display() from this function,
  it will be called by the main emulation engine.
***************************************************************************/
void trs80_vh_screenrefresh(struct osd_bitmap *bitmap, int full_refresh)
{
/* Special translation if video RAM with only 7 bits is present
 * I don't know if it's entirely correct, but it's close ;-)
 */
static UINT8 translate_videoram[2][256] = {
	{
		0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f,
		0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1a,0x1b,0x1c,0x1d,0x1e,0x1f,
		0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2a,0x2b,0x2c,0x2d,0x2e,0x2f,
		0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,0x3d,0x3e,0x3f,
		0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,
		0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5a,0x5b,0x5c,0x5d,0x5e,0x5f,
		0x60,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6a,0x6b,0x6c,0x6d,0x6e,0x6f,
		0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7a,0x7b,0x7c,0x7d,0x7e,0x7f,
		0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8a,0x8b,0x8c,0x8d,0x8e,0x8f,
		0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9a,0x9b,0x9c,0x9d,0x9e,0x9f,
		0xa0,0xa1,0xa2,0xa3,0xa4,0xa5,0xa6,0xa7,0xa8,0xa9,0xaa,0xab,0xac,0xad,0xae,0xaf,
		0xb0,0xb1,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xba,0xbb,0xbc,0xbd,0xbe,0xbf,
		0xc0,0xc1,0xc2,0xc3,0xc4,0xc5,0xc6,0xc7,0xc8,0xc9,0xca,0xcb,0xcc,0xcd,0xce,0xcf,
		0xd0,0xd1,0xd2,0xd3,0xd4,0xd5,0xd6,0xd7,0xd8,0xd9,0xda,0xdb,0xdc,0xdd,0xde,0xdf,
		0xe0,0xe1,0xe2,0xe3,0xe4,0xe5,0xe6,0xe7,0xe8,0xe9,0xea,0xeb,0xec,0xed,0xee,0xef,
		0xf0,0xf1,0xf2,0xf3,0xf4,0xf5,0xf6,0xf7,0xf8,0xf9,0xfa,0xfb,0xfc,0xfd,0xfe,0xff
	},
	{
		0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,
		0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5a,0x5b,0x5c,0x5d,0x5e,0x5f,
        0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2a,0x2b,0x2c,0x2d,0x2e,0x2f,
        0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,0x3d,0x3e,0x3f,
		0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,
        0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5a,0x5b,0x5c,0x5d,0x5e,0x5f,
		0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,
        0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5a,0x5b,0x5c,0x5d,0x5e,0x5f,
        0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8a,0x8b,0x8c,0x8d,0x8e,0x8f,
        0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9a,0x9b,0x9c,0x9d,0x9e,0x9f,
        0xa0,0xa1,0xa2,0xa3,0xa4,0xa5,0xa6,0xa7,0xa8,0xa9,0xaa,0xab,0xac,0xad,0xae,0xaf,
        0xb0,0xb1,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xba,0xbb,0xbc,0xbd,0xbe,0xbf,
        0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8a,0x8b,0x8c,0x8d,0x8e,0x8f,
        0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9a,0x9b,0x9c,0x9d,0x9e,0x9f,
        0xa0,0xa1,0xa2,0xa3,0xa4,0xa5,0xa6,0xa7,0xa8,0xa9,0xaa,0xab,0xac,0xad,0xae,0xaf,
        0xb0,0xb1,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xba,0xbb,0xbc,0xbd,0xbe,0xbf}
};

	int offs = 0;
	int translate = (input_port_0_r(0) & 0x40) ? 1 : 0;

	if( readinputport(0) & 0x08 )
        cpu_set_nmi_line(0, PULSE_LINE);

    /* check for changed color settings */
	if( color != (readinputport(0) & 3) )
    {
		color = readinputport(0) & 3;
		full_refresh = 1;
    }

    /* draw entire scrbitmap because of usrintrf functions
	 * called osd_clearbitmap or color change / scanline change
	 */
	if( palette_recalc() || full_refresh )
		memset(dirtybuffer, 1, videoram_size);

	/* do we have double width characters enabled ? */
	if( port_ff & 0x08 )
	{
		/* for every second character in the Video RAM, check if it has
		   been modified since last time and update it accordingly. */
		for (offs = videoram_size - 2; offs >= 0; offs -= 2)
		{
			if (dirtybuffer[offs])
			{
				int sx, sy;

				dirtybuffer[offs] = 0;
				sx = (offs % 64) * FW * 2;
				sy = (offs / 64) * FH;
				drawgfx(bitmap, Machine->gfx[1], translate_videoram[translate][videoram[offs]],
					color,0,0, sx,sy, &Machine->visible_area, TRANSPARENCY_NONE,0);
			}
		}
	}
	else
	/* normal width characters */
	{
		/* for every character in the Video RAM, check if it has
		* been modified since last time and update it accordingly. */
		for (offs = videoram_size - 1; offs >= 0; offs--)
		{
			if (dirtybuffer[offs])
			{
				int sx, sy;

                dirtybuffer[offs] = 0;
				sx = (offs % 64) * FW;
				sy = (offs / 64) * FH;
				drawgfx(bitmap, Machine->gfx[0], translate_videoram[translate][videoram[offs]],
					color,0,0, sx,sy, &Machine->visible_area, TRANSPARENCY_NONE,0);
            }
		}
	}
}
